@@ -20,10 +20,10 @@
 #include "SensorFaceController.h"
 #include "SensorFaceController_p.h"
 #include "SensorFace_p.h"
+#include <Sensor.h>
 #include <SensorQuery.h>
 
 #include <QtQml>
-#include <QDebug>
 
 #include <KDesktopFile>
 #include <KDeclarative/ConfigPropertyMap>
@@ -31,6 +31,11 @@
 #include <KLocalizedString>
 #include <KConfigLoader>
 #include <KPluginMetaData>
+#include <Solid/Block>
+#include <Solid/Device>
+#include <Solid/Predicate>
+#include <Solid/StorageAccess>
+#include <Solid/StorageVolume>
 
 using namespace KSysGuard;
 
@@ -105,7 +110,7 @@ void PresetsModel::reload()
         // TODO config
         QVariantMap config;
 
-        KConfigGroup configGroup(KSharedConfig::openConfig(p.filePath("config", QStringLiteral("faceproperties"))), QStringLiteral("Config"));
+        KConfigGroup configGroup(KSharedConfig::openConfig(p.filePath("config", QStringLiteral("faceproperties")), KConfig::SimpleConfig), QStringLiteral("Config"));
 
         const QStringList keys = configGroup.keyList();
         for (const QString &key : keys) {
@@ -133,6 +138,8 @@ QHash<int, QByteArray> PresetsModel::roleNames() const
 }
 
 QVector<QPair<QRegularExpression, QString>> KSysGuard::SensorFaceControllerPrivate::sensorIdReplacements;
+QRegularExpression SensorFaceControllerPrivate::oldDiskSensor = QRegularExpression(QStringLiteral("^disk\\/(.+)_\\(\\d+:\\d+\\)"));
+QRegularExpression SensorFaceControllerPrivate::oldPartitionSensor = QRegularExpression(QStringLiteral("^partitions(\\/.+)\\/"));
 
 SensorFaceControllerPrivate::SensorFaceControllerPrivate()
 {
@@ -151,10 +158,72 @@ SensorFaceControllerPrivate::SensorFaceControllerPrivate()
             { QRegularExpression(QStringLiteral("(.*)/transmitter/data$")), QStringLiteral("\\1/upload")},
             { QRegularExpression(QStringLiteral("(.*)/receiver/dataTotal$")), QStringLiteral("\\1/totalDownload")},
             { QRegularExpression(QStringLiteral("(.*)/transmitter/dataTotal$")), QStringLiteral("\\1/totalUpload")},
+            { QRegularExpression(QStringLiteral("(.*)/Rate/rio")), QStringLiteral("\\1/read")},
+            { QRegularExpression(QStringLiteral("(.*)/Rate/wio$")), QStringLiteral("\\1/write")},
+            { QRegularExpression(QStringLiteral("(.*)/freespace$")), QStringLiteral("\\1/free")},
+            { QRegularExpression(QStringLiteral("(.*)/filllevel$")), QStringLiteral("\\1/usedPercent")},
+            { QRegularExpression(QStringLiteral("(.*)/usedspace$")), QStringLiteral("\\1/used")},
+            { QRegularExpression(QStringLiteral("cpu/system/(.*)$")), QStringLiteral("cpu/all/\\1")},
+            { QRegularExpression(QStringLiteral("cpu/(.*)/sys$")), QStringLiteral("cpu/\\1/system")},
+            { QRegularExpression(QStringLiteral("cpu/(.*)/TotalLoad$")), QStringLiteral("cpu/\\1/usage")},
+            { QRegularExpression(QStringLiteral("cpu/cpu(\\d+)/clock$")), QStringLiteral("cpu/cpu\\1/frequency")},
+            { QRegularExpression(QStringLiteral("mem/(.*)level")), QStringLiteral("mem/\\1Percent")},
+            { QRegularExpression(QStringLiteral("mem/physical/allocated")), QStringLiteral("memory/physical/used")},
+            { QRegularExpression(QStringLiteral("mem/physical/available")), QStringLiteral("memory/physical/free")},
+            { QRegularExpression(QStringLiteral("mem/physical/buf")), QStringLiteral("memory/physical/buffer")},
+            { QRegularExpression(QStringLiteral("mem/physical/cached")), QStringLiteral("memory/physical/cache")},
+            { QRegularExpression(QStringLiteral("^mem/(.*)")), QStringLiteral("memory/\\1")},
+            { QRegularExpression(QStringLiteral("nvidia/(.*)/temperature$")), QStringLiteral("gpu/\\1/temperature")},
+            { QRegularExpression(QStringLiteral("nvidia/(.*)/memoryClock$")), QStringLiteral("gpu/\\1/memoryFrequency")},
+            { QRegularExpression(QStringLiteral("nvidia/(.*)/processorClock$")), QStringLiteral("gpu/\\1/coreFrequency")},
+            { QRegularExpression(QStringLiteral("nvidia/(.*)/(memory|sharedMemory)$")), QStringLiteral("gpu/\\1/usedVram")},
+            { QRegularExpression(QStringLiteral("nvidia/(.*)/(encoderUsage|decoderUsage)$")), QStringLiteral("gpu/\\1/usage")},
         };
     }
 }
 
+QString SensorFaceControllerPrivate::replaceDiskId(const QString &entryName) const
+{
+    const auto match = oldDiskSensor.match(entryName);
+    if (!match.hasMatch()) {
+        return entryName;
+    }
+    const QString device = match.captured(1);
+    Solid::Predicate predicate(Solid::DeviceInterface::StorageAccess);
+    predicate &= Solid::Predicate(Solid::DeviceInterface::Block, QStringLiteral("device"), QStringLiteral("/dev/%1").arg(device));
+    const auto devices = Solid::Device::listFromQuery(predicate);
+    if (devices.empty()) {
+        return QString();
+    }
+    QString sensorId = entryName;
+    const auto volume = devices[0].as<Solid::StorageVolume>();
+    const QString id = volume->uuid().isEmpty() ? volume->label() : volume->uuid();
+    return sensorId.replace(match.captured(0), QStringLiteral("disk/") + id);
+}
+
+QString SensorFaceControllerPrivate::replacePartitionId(const QString &entryName) const
+{
+    const auto match = oldPartitionSensor.match(entryName);
+    if (!match.hasMatch()) {
+        return entryName;
+    }
+    QString sensorId = entryName;
+
+    if (match.captured(1) == QLatin1String("/all")) {
+        return sensorId.replace(match.captured(0), QStringLiteral("disk/all/"));
+    }
+
+    const QString filePath = match.captured(1) == QLatin1String("/__root__") ? QStringLiteral("/") : match.captured(1);
+    const Solid::Predicate predicate(Solid::DeviceInterface::StorageAccess, QStringLiteral("filePath"), filePath);
+    const auto devices = Solid::Device::listFromQuery(predicate);
+    if (devices.empty()) {
+        return entryName;
+    }
+    const auto volume = devices[0].as<Solid::StorageVolume>();
+    const QString id = volume->uuid().isEmpty() ? volume->label() : volume->uuid();
+    return sensorId.replace(match.captured(0), QStringLiteral("disk/%1/").arg(id));
+}
+
 QJsonArray SensorFaceControllerPrivate::readSensors(const KConfigGroup &read, const QString &entryName)
 {
     auto original = QJsonDocument::fromJson(read.readEntry(entryName, QString()).toUtf8()).array();
@@ -167,6 +236,8 @@ QJsonArray SensorFaceControllerPrivate::readSensors(const KConfigGroup &read, co
                 sensorId.replace(replacement.first, replacement.second);
             }
         }
+        sensorId = replaceDiskId(sensorId);
+        sensorId = replacePartitionId(sensorId);
         newSensors.append(sensorId);
     }
 
@@ -187,22 +258,32 @@ QJsonArray SensorFaceControllerPrivate::readAndUpdateSensors(KConfigGroup& confi
     return newSensors;
 }
 
-QJsonArray SensorFaceControllerPrivate::resolveSensors(const QJsonArray &partialEntries)
+void SensorFaceControllerPrivate::resolveSensors(const QJsonArray &partialEntries, std::function<void(const QJsonArray&)> callback)
 {
-    QJsonArray sensors;
+    if (partialEntries.isEmpty()) {
+        callback(partialEntries);
+        return;
+    }
+
+    auto sensors = std::make_shared<QJsonArray>();
 
     for (const auto &id : partialEntries) {
-        KSysGuard::SensorQuery query{id.toString()};
-        query.execute();
-        query.waitForFinished();
-        auto ids = query.sensorIds();
-        std::stable_sort(ids.begin(), ids.end());
-        for (const auto &fitleredId : qAsConst(ids)) {
-            sensors.append(QJsonValue(fitleredId));
-        }
+        auto query = new KSysGuard::SensorQuery{id.toString()};
+        query->connect(query, &KSysGuard::SensorQuery::finished, q, [this, query, sensors, callback] {
+            query->sortByName();
+            const auto ids = query->sensorIds();
+            delete query;
+            std::transform(ids.begin(), ids.end(), std::back_inserter(*sensors), [] (const QString &id) {
+                return QJsonValue(id);
+            });
+            if (sensors.use_count() == 1) {
+                // We are the last query
+                callback(*sensors);
+            }
+        });
+        query->execute();
     }
-    return sensors;
-};
+}
 
 SensorFace *SensorFaceControllerPrivate::createGui(const QString &qmlPath)
 {
@@ -287,9 +368,18 @@ SensorFaceController::SensorFaceController(KConfigGroup &config, QQmlEngine *eng
 
     d->contextObj = new KLocalizedContext(this);
 
-    d->totalSensors = d->resolveSensors(d->readAndUpdateSensors(d->sensorsGroup, QStringLiteral("totalSensors")));
-    d->lowPrioritySensorIds = d->resolveSensors(d->readAndUpdateSensors(d->sensorsGroup, QStringLiteral("lowPrioritySensorIds")));
-    d->highPrioritySensorIds = d->resolveSensors(d->readAndUpdateSensors(d->sensorsGroup, QStringLiteral("highPrioritySensorIds")));
+    d->resolveSensors(d->readAndUpdateSensors(d->sensorsGroup, QStringLiteral("totalSensors")), [this] (const QJsonArray &resolvedSensors) {
+        d->totalSensors = resolvedSensors;
+        Q_EMIT totalSensorsChanged();
+    });
+    d->resolveSensors(d->readAndUpdateSensors(d->sensorsGroup, QStringLiteral("lowPrioritySensorIds")), [this] (const QJsonArray &resolvedSensors) {
+        d->lowPrioritySensorIds = resolvedSensors;
+        Q_EMIT lowPrioritySensorIdsChanged();
+    });
+    d->resolveSensors(d->readAndUpdateSensors(d->sensorsGroup, QStringLiteral("highPrioritySensorIds")), [this] (const QJsonArray &resolvedSensors) {
+        d->highPrioritySensorIds = resolvedSensors;
+        Q_EMIT highPrioritySensorIdsChanged();
+    });
 
     setFaceId(d->appearanceGroup.readEntry("chartFace", QStringLiteral("org.kde.ksysguard.piechart")));
 }
@@ -300,7 +390,13 @@ SensorFaceController::~SensorFaceController()
 
 QString SensorFaceController::title() const
 {
-    return d->appearanceGroup.readEntry("title", name());
+    // both Title and title can exist to allow i18n of Title
+    if (d->appearanceGroup.hasKey("title")) {
+        return d->appearanceGroup.readEntry("title");
+    } else {
+        // if neither exist fall back to name
+        return d->appearanceGroup.readEntry("Title", name());
+    }
 }
 
 void SensorFaceController::setTitle(const QString &title)
@@ -315,6 +411,24 @@ void SensorFaceController::setTitle(const QString &title)
     emit titleChanged();
 }
 
+bool SensorFaceController::showTitle() const
+{
+    return d->appearanceGroup.readEntry("showTitle", true);
+}
+
+void SensorFaceController::setShowTitle(bool show)
+{
+    if (show == showTitle()) {
+        return;
+    }
+
+    d->appearanceGroup.writeEntry("showTitle", show);
+    d->syncTimer->start();
+
+    emit showTitleChanged();
+}
+
+
 QJsonArray SensorFaceController::totalSensors() const
 {
     return d->totalSensors;
@@ -322,16 +436,16 @@ QJsonArray SensorFaceController::totalSensors() const
 
 void  SensorFaceController::setTotalSensors(const QJsonArray &totalSensors)
 {
-    QJsonArray resolvedSensors = d->resolveSensors(totalSensors);
+    d->resolveSensors(totalSensors, [this, totalSensors] (const QJsonArray &resolvedSensors) {
+        if (resolvedSensors == d->totalSensors) {
+            return;
+        }
+        d->totalSensors = resolvedSensors;
 
-    if (resolvedSensors == d->totalSensors) {
-        return;
-    }
-    d->totalSensors = resolvedSensors;
-
-    d->sensorsGroup.writeEntry("totalSensors", QJsonDocument(totalSensors).toJson(QJsonDocument::Compact));
-    d->syncTimer->start();
-    emit totalSensorsChanged();
+        d->sensorsGroup.writeEntry("totalSensors", QJsonDocument(totalSensors).toJson(QJsonDocument::Compact));
+        d->syncTimer->start();
+        emit totalSensorsChanged();
+    });
 }
 
 QJsonArray SensorFaceController::highPrioritySensorIds() const
@@ -342,16 +456,16 @@ QJsonArray SensorFaceController::highPrioritySensorIds() const
 
 void SensorFaceController::setHighPrioritySensorIds(const QJsonArray &highPrioritySensorIds)
 {
-    QJsonArray resolvedSensors = d->resolveSensors(highPrioritySensorIds);
+    d->resolveSensors(highPrioritySensorIds, [this, highPrioritySensorIds] (const QJsonArray &resolvedSensors) {
+        if (resolvedSensors == d->highPrioritySensorIds) {
+            return;
+        }
+        d->highPrioritySensorIds = resolvedSensors;
 
-    if (resolvedSensors == d->highPrioritySensorIds) {
-        return;
-    }
-    d->highPrioritySensorIds = resolvedSensors;
-
-    d->sensorsGroup.writeEntry("highPrioritySensorIds", QJsonDocument(highPrioritySensorIds).toJson(QJsonDocument::Compact));
-    d->syncTimer->start();
-    emit highPrioritySensorIdsChanged();
+        d->sensorsGroup.writeEntry("highPrioritySensorIds", QJsonDocument(highPrioritySensorIds).toJson(QJsonDocument::Compact));
+        d->syncTimer->start();
+        emit highPrioritySensorIdsChanged();
+    });
 }
 
 QVariantMap SensorFaceController::sensorColors() const
@@ -392,15 +506,16 @@ QJsonArray SensorFaceController::lowPrioritySensorIds() const
 
 void SensorFaceController::setLowPrioritySensorIds(const QJsonArray &lowPrioritySensorIds)
 {
-    QJsonArray resolvedSensors = d->resolveSensors(lowPrioritySensorIds);
-    if (resolvedSensors == d->lowPrioritySensorIds) {
-        return;
-    }
-    d->lowPrioritySensorIds = lowPrioritySensorIds;
+    d->resolveSensors(lowPrioritySensorIds, [this, lowPrioritySensorIds] (const QJsonArray &resolvedSensors) {
+        if (resolvedSensors == d->lowPrioritySensorIds) {
+            return;
+        }
+        d->lowPrioritySensorIds = lowPrioritySensorIds;
 
-    d->sensorsGroup.writeEntry("lowPrioritySensorIds", QJsonDocument(lowPrioritySensorIds).toJson(QJsonDocument::Compact));
-    d->syncTimer->start();
-    emit lowPrioritySensorIdsChanged();
+        d->sensorsGroup.writeEntry("lowPrioritySensorIds", QJsonDocument(lowPrioritySensorIds).toJson(QJsonDocument::Compact));
+        d->syncTimer->start();
+        emit lowPrioritySensorIdsChanged();
+    });
 }
 
 // from face config, immutable by the user
@@ -473,7 +588,7 @@ void SensorFaceController::setFaceId(const QString &face)
 
     d->contextObj->setTranslationDomain(QLatin1String("ksysguard_face_") + face);
 
-    d->faceProperties = KConfigGroup(KSharedConfig::openConfig(d->facePackage.filePath("FaceProperties")), QStringLiteral("Config"));
+    d->faceProperties = KConfigGroup(KSharedConfig::openConfig(d->facePackage.filePath("FaceProperties"), KConfig::SimpleConfig), QStringLiteral("Config"));
 
     const QString xmlPath = d->facePackage.filePath("mainconfigxml");
 
@@ -574,8 +689,11 @@ QQuickItem *SensorFaceController::sensorsConfigUi()
         return d->sensorsConfigUi;
     }
 
-    d->sensorsConfigUi = d->createConfigUi(QStringLiteral(":/ConfigSensors.qml"), {{QStringLiteral("controller"), QVariant::fromValue(this)}});
-
+    if (d->faceProperties.readEntry("SupportsSensors", true)) {
+        d->sensorsConfigUi = d->createConfigUi(QStringLiteral(":/ConfigSensors.qml"),{{QStringLiteral("controller"), QVariant::fromValue(this)}});
+    } else {
+        d->sensorsConfigUi = new QQuickItem;
+    }
     return d->sensorsConfigUi;
 }
 
@@ -606,17 +724,23 @@ void SensorFaceController::reloadConfig()
         d->faceConfigLoader->load();
     }
 
-    d->totalSensors = d->resolveSensors(d->readAndUpdateSensors(d->sensorsGroup, QStringLiteral("totalSensors")));
-    d->lowPrioritySensorIds = d->resolveSensors(d->readAndUpdateSensors(d->sensorsGroup, QStringLiteral("lowPrioritySensorIds")));
-    d->highPrioritySensorIds = d->resolveSensors(d->readAndUpdateSensors(d->sensorsGroup, QStringLiteral("highPrioritySensorIds")));
+    d->resolveSensors(d->readAndUpdateSensors(d->sensorsGroup, QStringLiteral("totalSensors")), [this] (const QJsonArray &resolvedSensors) {
+        d->totalSensors = resolvedSensors;
+        Q_EMIT totalSensorsChanged();
+    });
+    d->resolveSensors(d->readAndUpdateSensors(d->sensorsGroup, QStringLiteral("lowPrioritySensorIds")), [this] (const QJsonArray &resolvedSensors) {
+        d->lowPrioritySensorIds = resolvedSensors;
+        Q_EMIT lowPrioritySensorIdsChanged();
+    });
+    d->resolveSensors(d->readAndUpdateSensors(d->sensorsGroup, QStringLiteral("highPrioritySensorIds")), [this] (const QJsonArray &resolvedSensors) {
+        d->highPrioritySensorIds = resolvedSensors;
+        Q_EMIT highPrioritySensorIdsChanged();
+    });
 
     //Force to re-read all the values
     setFaceId(d->appearanceGroup.readEntry("chartFace", QStringLiteral("org.kde.ksysguard.textonly")));
     titleChanged();
-    totalSensorsChanged();
-    highPrioritySensorIdsChanged();
     sensorColorsChanged();
-    lowPrioritySensorIdsChanged();
 }
 
 void SensorFaceController::loadPreset(const QString &preset)
@@ -639,7 +763,7 @@ void SensorFaceController::loadPreset(const QString &preset)
 
     KDesktopFile df(presetPackage.path() + QStringLiteral("metadata.desktop"));
 
-    auto c = KSharedConfig::openConfig(presetPackage.filePath("config", QStringLiteral("faceproperties")));
+    auto c = KSharedConfig::openConfig(presetPackage.filePath("config", QStringLiteral("faceproperties")), KConfig::SimpleConfig);
     const KConfigGroup presetGroup(c, QStringLiteral("Config"));
     const KConfigGroup colorsGroup(c, QStringLiteral("SensorColors"));
 
@@ -662,7 +786,7 @@ void SensorFaceController::loadPreset(const QString &preset)
     emit sensorColorsChanged();
 
     if (d->faceConfigLoader) {
-        KConfigGroup presetGroup(KSharedConfig::openConfig(presetPackage.filePath("FaceProperties")), QStringLiteral("FaceConfig"));
+        KConfigGroup presetGroup(KSharedConfig::openConfig(presetPackage.filePath("FaceProperties"), KConfig::SimpleConfig), QStringLiteral("FaceConfig"));
 
         for (const QString &key : presetGroup.keyList()) {
             KConfigSkeletonItem *item = d->faceConfigLoader->findItemByName(key);
