@@ -56,10 +56,18 @@ SensorAgent::~SensorAgent()
 void SensorAgent::sendRequest( const QString &req, SensorClient *client, int id )
 {
   SensorRequest *sensorreq = nullptr;
-  SensorRequest nRequest { req, client, id };
-  if (mUnderwayRequests.contains(nRequest))
-    return;
-  mUnderwayRequests.insert(nRequest);
+  for(int i =0, total = mInputFIFO.size(); i < total; ++i) {
+    sensorreq = mInputFIFO.at(i);
+    if(id == sensorreq->id() && client == sensorreq->client() && req == sensorreq->request()) {
+      executeCommand();
+      return; //don't bother to resend the same request if we already have it in our queue to send
+    }
+  }
+  for(int i =0, total = mProcessingFIFO.size(); i < total; ++i) {
+    sensorreq = mProcessingFIFO.at(i);
+    if(id == sensorreq->id() && client == sensorreq->client() && req == sensorreq->request())
+      return; //don't bother to resend the same request if we have already sent the request to client and just waiting for an answer
+  }
 
   /* The request is registered with the FIFO so that the answer can be
    * routed back to the requesting client. */
@@ -154,13 +162,12 @@ void SensorAgent::processAnswer( const char *buf, int buflen )
 	// remove pending request from FIFO
 	if ( mProcessingFIFO.isEmpty() ) {
         qCDebug(LIBKSYSGUARD_KSGRD)	<< "ERROR: Received answer but have no pending "
-				<< "request!" << endl;
+                                << "request!";
 		mAnswerBuffer.clear();
 		continue;
 	}
 		
 	SensorRequest *req = mProcessingFIFO.dequeue();
-	mUnderwayRequests.remove(*req);
 	// we are now responsible for the memory of req - we must delete it!
 	if ( !req->client() ) {
 		/* The client has disappeared before receiving the answer
