@@ -47,6 +47,7 @@ public:
 
     QHash<QString, SensorInfo> sensorInfos;
     QHash<QString, QVariant> sensorData;
+    QVariantMap sensorColors;
 
     bool usedByQml = false;
     bool componentComplete = false;
@@ -125,6 +126,13 @@ QVariant SensorDataModel::data(const QModelIndex &index, int role) const
         return info.variantType;
     case SensorId:
         return sensor;
+    case Color:
+        if (!d->sensorColors.empty()) {
+            return d->sensorColors.value(sensor);
+        }
+    case UpdateInterval:
+        //TODO: Make this dynamic once the backend supports it.
+        return BackendUpdateInterval;
     default:
         break;
     }
@@ -166,6 +174,9 @@ QVariant SensorDataModel::headerData(int section, Qt::Orientation orientation, i
         return info.max;
     case Type:
         return info.variantType;
+    case UpdateInterval:
+        //TODO: Make this dynamic once the backend supports it.
+        return BackendUpdateInterval;
     default:
         break;
     }
@@ -227,6 +238,7 @@ void SensorDataModel::setSensors(const QStringList &sensorIds)
     if (!d->usedByQml || d->componentComplete) {
         d->sensorsChanged();
     }
+    Q_EMIT readyChanged();
     Q_EMIT sensorsChanged();
 }
 
@@ -252,6 +264,24 @@ void SensorDataModel::setEnabled(bool newEnabled)
     Q_EMIT enabledChanged();
 }
 
+QVariantMap SensorDataModel::sensorColors() const
+{
+    return d->sensorColors;
+}
+
+void SensorDataModel::setSensorColors(const QVariantMap &sensorColors)
+{
+    if (sensorColors == d->sensorColors) {
+        return;
+    }
+    d->sensorColors = sensorColors;
+    Q_EMIT sensorColorsChanged();
+}
+
+bool KSysGuard::SensorDataModel::isReady() const
+{
+    return d->sensors.size() == d->requestedSensors.size();
+}
 
 void SensorDataModel::addSensor(const QString &sensorId)
 {
@@ -358,6 +388,7 @@ void SensorDataModel::onMetaDataChanged(const QString &sensorId, const SensorInf
 
     SensorDaemonInterface::instance()->requestValue(sensorId);
     emit sensorMetaDataChanged();
+    Q_EMIT readyChanged();
 }
 
 void SensorDataModel::onValueChanged(const QString &sensorId, const QVariant &value)
